"""Contains tools to connect to cloud services."""

import logging

import requests
from pydantic import BaseModel


class FunctionError(BaseModel):
    """A representation of a function error data."""

    function_name: str
    function_file: str
    started_at: int
    finished_at: int
    error: str


class CloudLogger:
    """A class to log data to the cloud."""

    def __init__(self, api_key, dev_mode=False):
        """Set the API and synchronization variables."""
        if api_key is None:
            raise RuntimeError(
                "Authentication needed. Generate a key at: "
                "https://usefulmachines.dev/"
            )

        self.api_key = api_key
        self.API_ENDPOINT = (
            "https://lite.api.usefulmachines.dev/dev/error"
            if dev_mode
            else "https://lite.api.usefulmachines.dev/dev/error"
        )

    def upload_error(self, data: FunctionError):
        """Upload a function error to the cloud."""
        logging.info(f"[Useful] Upload error called with data: {data}")
        requests.post(
            f"{self.API_ENDPOINT}",
            data=data.model_dump_json(exclude_none=True),
            headers={"x-api-key": self.api_key},
            timeout=3,
        )


class FakeCloudLogger:
    """A fake cloud logger that does nothing but uploads data to a list attribute."""

    def __init__(self):
        """Initialize the FakeCloudLogger."""
        self.all_data = []

    def upload_error(self, data):
        """Simulate an function error upload to the cloud."""
        logging.info(f"[Useful] Upload error called with data: {data}")
        self.all_data.append(data.model_dump_json(exclude_none=True))



"""Main module."""

import inspect
import logging
import os
import time
import traceback

from useful.connections import CloudLogger, FunctionError


def __get_current_micros():
    return int(time.time() * 1000000)


logger = None


def check(
    verbose=0,
):
    """
    Tracks the runtime of a function and sends email upon failure.

    Args:
        verbose (int, optional): The verbosity of the logging.

    Returns:
        The output of the underlying function.
    """
    logging.basicConfig(
        format="%(levelname)s:%(message)s",
        level=[logging.ERROR, logging.INFO, logging.DEBUG][verbose],
    )

    def decorate(func):
        """Decorate the function."""

        def useful_wrapper(*args, **kwargs):
            """Wrap the function."""
            global logger  # noqa: PLW0603
            if logger is None:
                logger = CloudLogger(api_key=os.environ.get("USEFUL_API_KEY"))
            # error handing
            try:
                started_at = __get_current_micros()
                return func(*args, **kwargs)
            except Exception as e:
                tb_lines = traceback.format_exc().splitlines()
                print(tb_lines)

                filtered_tb = [
                    line
                    for line in tb_lines
                    if "useful_wrapper" not in line
                    and "return func(*args, **kwargs)" not in line
                ]
                final_traceback = "\n".join(filtered_tb)
                # [end] function log here
                payload = FunctionError(
                    function_name=func.__name__,
                    function_file=inspect.getfile(func),
                    started_at=started_at,
                    finished_at=__get_current_micros(),
                    error=final_traceback,
                )
                print(payload)
                logger.upload_error(payload)
                raise e  # noqa: TRY201

        return useful_wrapper

    return decorate
